package es.carlosgarcia;

import java.util.ArrayList;
import java.util.Date;

import android.content.ContentResolver;
import android.database.Cursor;
import android.net.Uri;
import android.os.AsyncTask;
import android.util.Log;

/**
 * Ventana de test del ContentProvider
 * @author Carlos García. 
 * @see http://www.carlos-garcia.es
 */
public class GmailReadTask extends AsyncTask<Void, Void, ArrayList<Mail>> {  // <Params, Progress, Result> 
	private static final String URI_PREFIX = "content://gmail-ls/conversations/";
	private static final String[] CONVERSATION_PROYECTION = {"_id", "date"};
	private static final String[] MESSAGE_PROJECTION		= {"_id", "subject", "fromAddress", "body", "dateReceivedMs"};
	 
	private ContentResolver resolver;
	private IBackgroudTaskResultListener listener;
	private String email;
	private Date   fromTime;
    
	public GmailReadTask(ContentResolver resolver, IBackgroudTaskResultListener listener, String email, Date fromTime){
		this.resolver = resolver;
		this.listener = listener;
		this.email	  = email;
		this.fromTime = fromTime;
	}
	
	protected ArrayList<Mail> doInBackground(Void... params) {
		ArrayList<Mail> mails = new ArrayList<Mail>(32);
        boolean finish = false;
        
		try {
	        // Conversaciones ordenadas por fecha (de más reciente a más antigua)
	        Cursor	cCursor	= resolver.query(Uri.parse(URI_PREFIX + email), CONVERSATION_PROYECTION, "label:^i" , null, null);	
		    
	        while (cCursor.moveToNext() && (! finish)) {
        		finish = fromTime.before(new Date(cCursor.getLong(1)));

            	if (! finish){
            		// Obtenemos un cursor para esa conversación.
            		String conv_id  = cCursor.getString(cCursor.getColumnIndex("_id"));
            		Uri	   uri	    = Uri.parse(URI_PREFIX + email + "/" + conv_id + "/messages");
                    Cursor mCursor	= resolver.query(uri, MESSAGE_PROJECTION, null, null, null);	                
                    
                    while (mCursor.moveToNext() &&  (! finish)){
                    	long mtime = mCursor.getLong(4);
                    	finish = fromTime.before(new Date(mtime));
                    	if (! finish){
                    		mails.add(new Mail(mCursor.getString(0), mCursor.getString(2), mCursor.getString(1), mCursor.getString(4), mtime));                        		
                    	}
                    }                    	
            	}        		
	    	}
		} catch (Exception ex){	// TODO: Nota, capturar la excepción padre de todas las posibles 
			Log.e("GmailReadApp", ex.toString());
			mails.add(new Mail(null, null, ex.toString(), ex.toString(), 0));
		}
		
		return mails;
	}
	
	
	@Override
	protected void onPostExecute(ArrayList<Mail> result) {
		super.onPostExecute(result);
		this.listener.backgroundTaskFinished("ok", result);
	}

}

